"""Tests for systems.json files
"""

import unittest
import json
import os.path

# 3rd Party imports
from jsonschema import validate
from plugins.filters import is_safe

class TestSystemSyntax(unittest.TestCase):
    """Validates all plugins in `api/plugins/` using our JSON Schema.
    """
    # Get file paths relative to this file's known location
    schema_path = os.path.join(os.path.dirname(__file__), "../schemas/system-schema.json")
    system_files = [os.path.join(os.path.dirname(__file__), "../systems.json")]
    # Class variable holding the current JSON Schema. See setUpClass() for more info
    schema = None

    @classmethod
    def setUpClass(cls):
        """Open and read in the current plugin JSON Schema from disk for us during testing.
        """
        with open(cls.schema_path, mode='r') as f:
            cls.schema = json.load(f)
        cls.errors = []

    def tearDown(self):
        if len(self.errors) > 0:
            for err in self.errors:
                print("-"*70)
                try:
                    msg = err["error"].message
                except AttributeError:
                    msg = err["error"]
                print(f"Error validating system '{err['system']}': {msg} -- {err['file']}")
            self.assertIsNone(1)

    def _test_system_data(self, system, data, file):
        """Runs a single test against the provided system's data

        Args:
            system (str): Name of the system for the provided data
            data (dict): The systems data to test
            file (str): The name of the file data belongs to
        """
        try:
            self.assertIsNone(validate(instance=data, schema=self.schema))
        except Exception as e:
            self.errors.append({"system": system, "file": os.path.basename(file), "error": e})

    def test_isvalid(self):
        """For each systems file, validate each system in file against our schema. If an
        exception is raised, the plugin file is NOT valid.
        """
        for system_file in self.system_files:
            with open(system_file, mode='r') as f:
                systems = json.load(f).get("systems")
                if systems:
                    for system, system_data in systems.items():
                        self._test_system_data(system, system_data, system_file)
                else:
                    self.errors[system_file] = "Has no systems attribute."

    def test_issafe(self):
        """Ensure each system name and node type passes the is_safe constraint provided by plugins.filters"""
        for system_file in self.system_files:
            with open(system_file, mode='r') as f:
                systems = json.load(f).get("systems")
                if systems:
                    for system, system_data in systems.items():
                        if not is_safe(system):
                            self.errors.append({"system": system, "file": os.path.basename(system_file), "error": "System name fails 'is_safe' constraint"})
                        for node in system_data.get("nodes", []):
                            node_name = node.get("name")
                            if not is_safe(node_name):
                                self.errors.append({"system": system, "file": os.path.basename(system_file), "error": f"Node name '{node_name}' fails 'is_safe' constraint"})

